#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "LoadFlat.h"
#include "export.h"

extern unsigned char g_FlatFileAsDownloaded[];
asm ( ".SECTION sdram0; .INC/BINARY _g_FlatFileAsDownloaded[] = \"Debug/Example_2_-_DLM.dxe.bflt\"; .PREVIOUS;" );

#pragma align 0x10000
segment ("sdram0") unsigned char g_FlatFileDestinationText[ 16*1024 ];
#pragma align 0x10000
segment ("sdram0") unsigned char g_FlatFileDestinationData[ 16*1024 ];
#pragma align 0x10000
segment ("sdram0") unsigned char g_FlatFileDestinationBSS [ 16*1024 ];

// Reserve 1MB in SDRAM for the final destination of the flat file.

int vals[] = { 1, 1, 1, 2, 2, 2, 10, 10, 10, 10 };

int main ()
{
	FLT_RESULT ret;
	unsigned long dwSizeText, dwSizeData, dwSizeBSS, dwTotalSize;
	void *pText;
	
	int (*pf)(const int *, const unsigned int);
	int result;
	
	if ( ret = FLT_GetSizes ( g_FlatFileAsDownloaded, &dwSizeText, &dwSizeData, &dwSizeBSS ) )
		return (int) ret;
	
	// Allocate space for the relocated BLFT file.  Zero-fill the .bss section.
	// Note: destination location must not be in L1, because the memory must be
	// accessed as *both* code and data.  Placement in L1 will cause a sequence
	// exception or some kind since only one of those access types will be legal.
	// For this reason, we've created a static block in SDRAM (g_FlatFileDestination)
	
	memset ( g_FlatFileDestinationBSS, 0, dwSizeBSS );
	
	// "pText" is an out parameter representing the beginning of the text section.
	// If there is only a single exported functions (with no look-up table), the
	// address to jump to should be "pText".  If there is a look-up table, it should
	// be pointed to by "pText" (individual functions can be searched for with the
	// FLT_LookUpByName() API).

	if ( ret = FLT_RelocateFlat ( g_FlatFileAsDownloaded,
				g_FlatFileDestinationText,
				g_FlatFileDestinationData,
				g_FlatFileDestinationBSS,
				&pText ) ) return (int) ret;
				
	printf ( ".text at 0x%x\n.data at 0x%x\n.bss at 0x%x\nend at 0x%x\n", 
				g_FlatFileDestinationText,
				g_FlatFileDestinationData,
				g_FlatFileDestinationBSS,
				g_FlatFileDestinationBSS + dwSizeBSS );

	// Lookup the "mean" entry point and then call it.
	
	if ( ret = FLT_LookupByName ( pText, "mean", (void**) &pf ) ) return (int) ret;
	result = pf ( vals, sizeof (vals) / sizeof ( int ) );
	printf ( "Call to loaded module \"mean\" returned %d.\n", (int) result );
	
	// Do the same with "median".
	
	if ( ret = FLT_LookupByName ( pText, "median", (void**) &pf ) ) return (int) ret;
	result = pf ( vals, sizeof (vals) / sizeof ( int ) );
	printf ( "Call to loaded module \"median\" returned %d.\n", (int) result );
	
	// Now try to lookup an entry that does not exist.
	
	ret = FLT_LookupByName ( pText, "blahblahblah", (void**) &pf );
	printf ( "Lookup in loaded module for \"blahblahblah\" returned %d.\n", (int) ret );
	
	return 0;
}
